@@ -1,11 +1,9 @@
 /*
- * $Id: hashtable.h,v 3.1.2.7 1997/05/06 05:31:12 hzoli Exp $
- *
  * hashtable.h - header file for hash table handling code
  *
  * This file is part of zsh, the Z shell.
  *
- * Copyright (c) 1992-1997 Paul Falstad
+ * Copyright (c) 1992-1996 Paul Falstad
  * All rights reserved.
  *
  * Permission is hereby granted, without written agreement and without
@@ -80,8 +78,8 @@ struct iparam {
     void *value;
     void (*func1) _((void));	/* set func                              */
     char *(*func2) _((void));	/* get func                              */
-    void (*unsetfn) _((Param, int));	/* unset func                    */
     int ct;			/* output base or field width            */
+    void *data;			/* used by getfns                        */
     char *env;			/* location in environment, if exported  */
     char *ename;		/* name of corresponding environment var */
     Param old;			/* old struct for use with local         */
@@ -91,20 +89,20 @@ struct iparam {
 special_params[] ={
 #define SFN(X) BR(((void (*)_((Param, char *)))(X)))
 #define GFN(X) BR(((char *(*)_((Param)))(X)))
-#define IPDEF1(A,B,C,D) {NULL,A,PM_INTEGER|PM_SPECIAL|D,BR(NULL),SFN(C),GFN(B),stdunsetfn,10,NULL,NULL,NULL,0}
+#define IPDEF1(A,B,C,D) {NULL,A,PM_INTEGER|PM_SPECIAL|D,BR(NULL),SFN(C),GFN(B),10,NULL,NULL,NULL,NULL,0}
 IPDEF1("#", poundgetfn, nullsetfn, PM_READONLY),
 IPDEF1("ERRNO", errnogetfn, nullsetfn, PM_READONLY),
-IPDEF1("GID", gidgetfn, gidsetfn, PM_DONTIMPORT | PM_RESTRICTED),
-IPDEF1("EGID", egidgetfn, egidsetfn, PM_DONTIMPORT | PM_RESTRICTED),
-IPDEF1("HISTSIZE", histsizegetfn, histsizesetfn, PM_RESTRICTED),
+IPDEF1("GID", gidgetfn, gidsetfn, PM_DONTIMPORT),
+IPDEF1("EGID", egidgetfn, egidsetfn, PM_DONTIMPORT),
+IPDEF1("HISTSIZE", histsizegetfn, histsizesetfn, 0),
 IPDEF1("RANDOM", randomgetfn, randomsetfn, 0),
 IPDEF1("SECONDS", secondsgetfn, secondssetfn, 0),
-IPDEF1("UID", uidgetfn, uidsetfn, PM_DONTIMPORT | PM_RESTRICTED),
-IPDEF1("EUID", euidgetfn, euidsetfn, PM_DONTIMPORT | PM_RESTRICTED),
+IPDEF1("UID", uidgetfn, uidsetfn, PM_DONTIMPORT),
+IPDEF1("EUID", euidgetfn, euidsetfn, PM_DONTIMPORT),
 IPDEF1("TTYIDLE", ttyidlegetfn, nullsetfn, PM_READONLY),
 
-#define IPDEF2(A,B,C,D) {NULL,A,PM_SCALAR|PM_SPECIAL|D,BR(NULL),SFN(C),GFN(B),stdunsetfn,0,NULL,NULL,NULL,0}
-IPDEF2("USERNAME", usernamegetfn, usernamesetfn, PM_DONTIMPORT|PM_RESTRICTED),
+#define IPDEF2(A,B,C,D) {NULL,A,PM_SCALAR|PM_SPECIAL|D,BR(NULL),BR(C),BR(B),0,NULL,NULL,NULL,NULL,0}
+IPDEF2("USERNAME", usernamegetfn, usernamesetfn, PM_DONTIMPORT),
 IPDEF2("-", dashgetfn, nullsetfn, PM_READONLY),
 IPDEF2("histchars", histcharsgetfn, histcharssetfn, PM_DONTIMPORT),
 IPDEF2("HOME", homegetfn, homesetfn, 0),
@@ -113,41 +111,41 @@ IPDEF2("WORDCHARS", wordcharsgetfn, wordcharssetfn, 0),
 IPDEF2("IFS", ifsgetfn, ifssetfn, PM_DONTIMPORT),
 IPDEF2("_", underscoregetfn, nullsetfn, PM_READONLY),
 
-#ifdef LC_ALL
-#define LCIPDEF(name) IPDEF2(name, strgetfn, lcsetfn, PM_UNSET)
+#ifdef USE_LOCALE
+#define LCIPDEF(A,B) {NULL,A,PM_SCALAR|PM_SPECIAL|PM_UNSET,BR(NULL),BR(lcsetfn),BR(strgetfn),0,(void *)B,NULL,NULL,NULL,0}
 IPDEF2("LANG", strgetfn, langsetfn, PM_UNSET),
 IPDEF2("LC_ALL", strgetfn, lc_allsetfn, PM_UNSET),
 #ifdef LC_COLLATE
-LCIPDEF("LC_COLLATE"),
+LCIPDEF("LC_COLLATE", LC_COLLATE),
 #endif
 #ifdef LC_CTYPE
-LCIPDEF("LC_CTYPE"),
+LCIPDEF("LC_CTYPE", LC_CTYPE),
 #endif
 #ifdef LC_MESSAGES
-LCIPDEF("LC_MESSAGES"),
+LCIPDEF("LC_MESSAGES", LC_MESSAGES),
 #endif
 #ifdef LC_TIME
-LCIPDEF("LC_TIME"),
-#endif
+LCIPDEF("LC_TIME", LC_TIME),
 #endif
+#endif /* USE_LOCALE */
 
-#define IPDEF4(A,B) {NULL,A,PM_INTEGER|PM_READONLY|PM_SPECIAL,BR((void *)B),SFN(nullsetfn),GFN(intvargetfn),stdunsetfn,10,NULL,NULL,NULL,0}
+#define IPDEF4(A,B) {NULL,A,PM_INTEGER|PM_READONLY|PM_SPECIAL,BR(NULL),BR(nullsetfn),GFN(intvargetfn),10,(void *)B,NULL,NULL,NULL,0}
 IPDEF4("!", &lastpid),
 IPDEF4("$", &mypid),
 IPDEF4("?", &lastval),
 IPDEF4("LINENO", &lineno),
 IPDEF4("PPID", &ppid),
 
-#define IPDEF5(A,B,F) {NULL,A,PM_INTEGER|PM_SPECIAL,BR((void *)B),SFN(F),GFN(intvargetfn),stdunsetfn,10,NULL,NULL,NULL,0}
+#define IPDEF5(A,B,F) {NULL,A,PM_INTEGER|PM_SPECIAL,BR(NULL),SFN(F),GFN(intvargetfn),10,(void *)B,NULL,NULL,NULL,0}
 IPDEF5("COLUMNS", &columns, zlevarsetfn),
 IPDEF5("LINES", &lines, zlevarsetfn),
 IPDEF5("OPTIND", &zoptind, intvarsetfn),
 IPDEF5("SHLVL", &shlvl, intvarsetfn),
 
-#define IPDEF6(A,B) {NULL,A,PM_SCALAR|PM_READONLY|PM_SPECIAL,BR((void *)B),SFN(nullsetfn),GFN(strvargetfn),stdunsetfn,0,NULL,NULL,NULL,0}
+#define IPDEF6(A,B) {NULL,A,PM_SCALAR|PM_READONLY|PM_SPECIAL,BR(NULL),SFN(nullsetfn),BR(strvargetfn),0,(void *)B,NULL,NULL,NULL,0}
 IPDEF6("PWD", &pwd),
 
-#define IPDEF7(A,B) {NULL,A,PM_SCALAR|PM_SPECIAL,BR((void *)B),SFN(strvarsetfn),GFN(strvargetfn),stdunsetfn,0,NULL,NULL,NULL,0}
+#define IPDEF7(A,B) {NULL,A,PM_SCALAR|PM_SPECIAL,BR(NULL),BR(strvarsetfn),BR(strvargetfn),0,(void *)B,NULL,NULL,NULL,0}
 IPDEF7("OLDPWD", &oldpwd),
 IPDEF7("OPTARG", &zoptarg),
 IPDEF7("NULLCMD", &nullcmd),
@@ -162,22 +160,16 @@ IPDEF7("RPS1", &rprompt),
 IPDEF7("SPROMPT", &sprompt),
 IPDEF7("0", &argzero),
 
-#define IPDEF8(A,B,C,D) {NULL,A,D|PM_SCALAR|PM_SPECIAL,BR((void *)B),SFN(colonarrsetfn),GFN(colonarrgetfn),stdunsetfn,0,NULL,C,NULL,0}
-IPDEF8("CDPATH", &cdpath, "cdpath", 0),
-IPDEF8("FIGNORE", &fignore, "fignore", 0),
-IPDEF8("FPATH", &fpath, "fpath", 0),
-IPDEF8("MAILPATH", &mailpath, "mailpath", 0),
-IPDEF8("WATCH", &watch, "watch", 0),
-IPDEF8("PATH", &path, "path", PM_RESTRICTED),
-IPDEF8("PSVAR", &psvar, "psvar", 0),
+#define IPDEF8(A,B,C) {NULL,A,PM_SCALAR|PM_SPECIAL,BR(NULL),SFN(colonarrsetfn),GFN(colonarrgetfn),0,(void *)B,NULL,C,NULL,0}
+IPDEF8("CDPATH", &cdpath, "cdpath"),
+IPDEF8("FIGNORE", &fignore, "fignore"),
+IPDEF8("FPATH", &fpath, "fpath"),
+IPDEF8("MAILPATH", &mailpath, "mailpath"),
+IPDEF8("WATCH", &watch, "watch"),
+IPDEF8("PATH", &path, "path"),
+IPDEF8("PSVAR", &psvar, "psvar"),
 
-#ifdef DYNAMIC
-/* MODULE_PATH is not imported for security reasons */
-IPDEF8("MODULE_PATH", &module_path, "module_path", PM_DONTIMPORT|PM_RESTRICTED),
-#endif
-
-#define IPDEF9F(A,B,C,D) {NULL,A,D|PM_ARRAY|PM_SPECIAL|PM_DONTIMPORT,BR((void *)B),SFN(arrvarsetfn),GFN(arrvargetfn),stdunsetfn,0,NULL,C,NULL,0}
-#define IPDEF9(A,B,C) IPDEF9F(A,B,C,0)
+#define IPDEF9(A,B,C) {NULL,A,PM_ARRAY|PM_SPECIAL|PM_DONTIMPORT,BR(NULL),SFN(arrvarsetfn),GFN(arrvargetfn),0,(void *)B,NULL,C,NULL,0}
 IPDEF9("*", &pparams, NULL),
 IPDEF9("@", &pparams, NULL),
 {NULL, NULL},
@@ -192,20 +184,16 @@ IPDEF7("PROMPT", &prompt),
 IPDEF7("PROMPT2", &prompt2),
 IPDEF7("PROMPT3", &prompt3),
 IPDEF7("PROMPT4", &prompt4),
-IPDEF8("MANPATH", &manpath, "manpath", 0),
+IPDEF8("MANPATH", &manpath, "manpath"),
 IPDEF9("argv", &pparams, NULL),
 IPDEF9("fignore", &fignore, "FIGNORE"),
 IPDEF9("cdpath", &cdpath, "CDPATH"),
 IPDEF9("fpath", &fpath, "FPATH"),
 IPDEF9("mailpath", &mailpath, "MAILPATH"),
 IPDEF9("manpath", &manpath, "MANPATH"),
-IPDEF9("psvar", &psvar, "PSVAR"),
 IPDEF9("watch", &watch, "WATCH"),
-
-#ifdef DYNAMIC
-IPDEF9F("module_path", &module_path, "MODULE_PATH", PM_RESTRICTED),
-#endif
-IPDEF9F("path", &path, "PATH", PM_RESTRICTED),
+IPDEF9("path", &path, "PATH"),
+IPDEF9("psvar", &psvar, "PSVAR"),
 
 {NULL, NULL}
 };
@@ -268,10 +256,12 @@ struct builtin builtins[] =
     {NULL, "alias", BINF_MAGICEQUALS, bin_alias, 0, -1, 0, "Lgmr", NULL},
     {NULL, "autoload", BINF_TYPEOPTS, bin_functions, 0, -1, 0, "t", "u"},
     {NULL, "bg", 0, bin_fg, 0, -1, BIN_BG, NULL, NULL},
+    {NULL, "bindkey", 0, bin_bindkey, 0, -1, 0, "asvemdr", NULL},
     {NULL, "break", BINF_PSPECIAL, bin_break, 0, 1, BIN_BREAK, NULL, NULL},
     {NULL, "bye", 0, bin_break, 0, 1, BIN_EXIT, NULL, NULL},
     {NULL, "cd", 0, bin_cd, 0, 2, BIN_CD, NULL, NULL},
     {NULL, "chdir", 0, bin_cd, 0, 2, BIN_CD, NULL, NULL},
+    {NULL, "compctl", 0, bin_compctl, 0, -1, 0, NULL, NULL},
     {NULL, "continue", BINF_PSPECIAL, bin_break, 0, 1, BIN_CONTINUE, NULL, NULL},
     {NULL, "declare", BINF_TYPEOPTS | BINF_MAGICEQUALS | BINF_PSPECIAL, bin_typeset, 0, -1, 0, "LRUZfilrtux", NULL},
     {NULL, "dirs", 0, bin_dirs, 0, -1, 0, "v", NULL},
@@ -279,7 +269,7 @@ struct builtin builtins[] =
     {NULL, "disown", 0, bin_fg, 0, -1, BIN_DISOWN, NULL, NULL},
     {NULL, "echo", BINF_PRINTOPTS | BINF_ECHOPTS, bin_print, 0, -1, BIN_ECHO, "neE", "-"},
     {NULL, "echotc", 0, bin_echotc, 1, -1, 0, NULL, NULL},
-    {NULL, "emulate", 0, bin_emulate, 1, 1, 0, "R", NULL},
+    {NULL, "emulate", 0, bin_emulate, 1, 1, 0, "LR", NULL},
     {NULL, "enable", 0, bin_enable, 0, -1, BIN_ENABLE, "afmr", NULL},
     {NULL, "eval", BINF_PSPECIAL, bin_eval, 0, -1, BIN_EVAL, NULL, NULL},
     {NULL, "exit", BINF_PSPECIAL, bin_break, 0, 1, BIN_EXIT, NULL, NULL},
@@ -290,7 +280,7 @@ struct builtin builtins[] =
     {NULL, "functions", BINF_TYPEOPTS, bin_functions, 0, -1, 0, "mtu", NULL},
     {NULL, "getln", 0, bin_read, 0, -1, 0, "ecnAlE", "zr"},
     {NULL, "getopts", 0, bin_getopts, 2, -1, 0, NULL, NULL},
-    {NULL, "hash", BINF_MAGICEQUALS, bin_hash, 0, -1, 0, "dfmrv", NULL},
+    {NULL, "hash", BINF_MAGICEQUALS, bin_hash, 0, -1, 0, "dfmr", NULL},
 
 #ifdef ZSH_HASH_DEBUG
     {NULL, "hashinfo", 0, bin_hashinfo, 0, 0, 0, NULL, NULL},
@@ -298,9 +288,10 @@ struct builtin builtins[] =
 
     {NULL, "history", 0, bin_fc, 0, -1, BIN_FC, "nrdDfEim", "l"},
     {NULL, "integer", BINF_TYPEOPTS | BINF_MAGICEQUALS | BINF_PSPECIAL, bin_typeset, 0, -1, 0, "lrtux", "i"},
-    {NULL, "jobs", 0, bin_fg, 0, -1, BIN_JOBS, "dlpZrs", NULL},
+    {NULL, "jobs", 0, bin_fg, 0, -1, BIN_JOBS, "lpZrs", NULL},
     {NULL, "kill", 0, bin_kill, 0, -1, 0, NULL, NULL},
     {NULL, "let", 0, bin_let, 1, -1, 0, NULL, NULL},
+    {NULL, "limit", 0, bin_limit, 0, -1, 0, "sh", NULL},
     {NULL, "local", BINF_TYPEOPTS | BINF_MAGICEQUALS | BINF_PSPECIAL, bin_typeset, 0, -1, 0, "LRUZilrtu", NULL},
     {NULL, "log", 0, bin_log, 0, 0, 0, NULL, NULL},
     {NULL, "logout", 0, bin_break, 0, 1, BIN_LOGOUT, NULL, NULL},
@@ -309,19 +300,17 @@ struct builtin builtins[] =
     {NULL, "mem", 0, bin_mem, 0, 0, 0, "v", NULL},
 #endif
 
-#ifdef DYNAMIC
-    {NULL, "zmodload", 0, bin_zmodload, 0, -1, 0, "Laudi", NULL},
-#endif
     {NULL, "popd", 0, bin_cd, 0, 2, BIN_POPD, NULL, NULL},
     {NULL, "print", BINF_PRINTOPTS, bin_print, 0, -1, BIN_PRINT, "RDPnrslzNu0123456789pioOcm-", NULL},
     {NULL, "pushd", 0, bin_cd, 0, 2, BIN_PUSHD, NULL, NULL},
     {NULL, "pushln", BINF_PRINTOPTS, bin_print, 0, -1, BIN_PRINT, NULL, "-nz"},
-    {NULL, "pwd", 0, bin_pwd, 0, 0, 0, "rLP", NULL},
+    {NULL, "pwd", 0, bin_pwd, 0, 0, 0, "r", NULL},
     {NULL, "r", BINF_R, bin_fc, 0, -1, BIN_FC, "nrl", NULL},
     {NULL, "read", 0, bin_read, 0, -1, 0, "rzu0123456789pkqecnAlE", NULL},
     {NULL, "readonly", BINF_TYPEOPTS | BINF_MAGICEQUALS | BINF_PSPECIAL, bin_typeset, 0, -1, 0, "LRUZfiltux", "r"},
-    {NULL, "rehash", 0, bin_hash, 0, 0, 0, "dfv", "r"},
+    {NULL, "rehash", 0, bin_hash, 0, 0, 0, "df", "r"},
     {NULL, "return", BINF_PSPECIAL, bin_break, 0, 1, BIN_RETURN, NULL, NULL},
+    {NULL, "sched", 0, bin_sched, 0, -1, 0, NULL, NULL},
     {NULL, "set", BINF_PSPECIAL, bin_set, 0, -1, 0, NULL, NULL},
     {NULL, "setopt", 0, bin_setopt, 0, -1, BIN_SETOPT, NULL, NULL},
     {NULL, "shift", BINF_PSPECIAL, bin_shift, 0, -1, 0, NULL, NULL},
@@ -332,18 +321,21 @@ struct builtin builtins[] =
     {NULL, "times", BINF_PSPECIAL, bin_times, 0, 0, 0, NULL, NULL},
     {NULL, "trap", BINF_PSPECIAL, bin_trap, 0, -1, 0, NULL, NULL},
     {NULL, "true", 0, bin_true, 0, -1, 0, NULL, NULL},
-    {NULL, "type", 0, bin_whence, 0, -1, 0, "ampfs", "v"},
+    {NULL, "type", 0, bin_whence, 0, -1, 0, "ampf", "v"},
     {NULL, "typeset", BINF_TYPEOPTS | BINF_MAGICEQUALS | BINF_PSPECIAL, bin_typeset, 0, -1, 0, "LRUZfilrtuxm", NULL},
+    {NULL, "ulimit", 0, bin_ulimit, 0, -1, 0, NULL, NULL},
     {NULL, "umask", 0, bin_umask, 0, 1, 0, "S", NULL},
     {NULL, "unalias", 0, bin_unhash, 1, -1, 0, "m", "a"},
     {NULL, "unfunction", 0, bin_unhash, 1, -1, 0, "m", "f"},
     {NULL, "unhash", 0, bin_unhash, 1, -1, 0, "adfm", NULL},
+    {NULL, "unlimit", 0, bin_unlimit, 0, -1, 0, "hs", NULL},
     {NULL, "unset", BINF_PSPECIAL, bin_unset, 1, -1, 0, "fm", NULL},
     {NULL, "unsetopt", 0, bin_setopt, 0, -1, BIN_UNSETOPT, NULL, NULL},
+    {NULL, "vared", 0, bin_vared, 1, 7, 0, NULL, NULL},
     {NULL, "wait", 0, bin_fg, 0, -1, BIN_WAIT, NULL, NULL},
-    {NULL, "whence", 0, bin_whence, 0, -1, 0, "acmpvfs", NULL},
-    {NULL, "where", 0, bin_whence, 0, -1, 0, "pms", "ca"},
-    {NULL, "which", 0, bin_whence, 0, -1, 0, "amps", "c"},
+    {NULL, "whence", 0, bin_whence, 0, -1, 0, "acmpvf", NULL},
+    {NULL, "where", 0, bin_whence, 0, -1, 0, "pm", "ca"},
+    {NULL, "which", 0, bin_whence, 0, -1, 0, "amp", "c"},
     {NULL, NULL}
 };
 #else
