@@ -3,7 +3,7 @@
  *
  * This file is part of zsh, the Z shell.
  *
- * Copyright (c) 1992-1997 Paul Falstad
+ * Copyright (c) 1992-1996 Paul Falstad
  * All rights reserved.
  *
  * Permission is hereby granted, without written agreement and without
@@ -27,14 +27,17 @@
  *
  */
 
-#include "zsh.mdh"
-#include "mem.pro"
+#include "zsh.h"
+
+void *(*alloc) _((size_t));
+void *(*ncalloc) _((size_t));
 
 /*
+
 	There are two ways to allocate memory in zsh.  The first way is
-	to call zalloc/zshcalloc, which call malloc/calloc directly.  It
+	to call zalloc/zcalloc, which call malloc/calloc directly.  It
 	is legal to call realloc() or free() on memory allocated this way.
-	The second way is to call zhalloc/hcalloc, which allocates memory
+	The second way is to call halloc/hcalloc, which allocates memory
 	from one of the memory pools on the heap stack.  Such memory pools 
 	will automatically created when the heap allocation routines are
 	called.  To be sure that they are freed at appropriate times
@@ -54,39 +57,25 @@
 	Memory allocated in this way does not have to be freed explicitly;
 	it will all be freed when the pool is destroyed.  In fact,
 	attempting to free this memory may result in a core dump.
+	The pair of pointers ncalloc and alloc may point to either
+	zalloc & zcalloc or halloc & hcalloc; permalloc() sets them to the
+	former, and heapalloc() sets them to the latter. This can be useful.
+	For example, the dupstruct() routine duplicates a syntax tree,
+	allocating the new memory for the tree using alloc().  If you want
+	to duplicate a structure for a one-time use (i.e. to execute the list
+	in a for loop), call heapalloc(), then dupstruct().  If you want
+	to duplicate a structure in order to preserve it (i.e. a function
+	definition), call permalloc(), then dupstruct().
 
-	If possible, the heaps are allocated using mmap() so that the
-	(*real*) heap isn't filled up with empty zsh heaps. If mmap()
-	is not available and zsh's own allocator is used, we use a simple trick
-	to avoid that: we allocate a large block of memory before allocating
-	a heap pool, this memory is freed again immediately after the pool
-	is allocated. If there are only small blocks on the free list this
-	guarantees that the memory for the pool is at the end of the memory
-	which means that we can give it back to the system when the pool is
-	freed.
-
-	hrealloc(char *p, size_t old, size_t new) is an optimisation
-	with a similar interface to realloc().  Typically the new size
-	will be larger than the old one, since there is no gain in
-	shrinking the allocation (indeed, that will confused hrealloc()
-	since it will forget that the unused space once belonged to this
-	pointer).  However, new == 0 is a special case; then if we
-	had to allocate a special heap for this memory it is freed at
-	that point.
+	If we use zsh's own allocator we use a simple trick to avoid that
+	the (*real*) heap fills up with empty zsh-heaps: we allocate a
+	large block of memory before allocating a heap pool, this memory
+	is freed again immediately after the pool is allocated. If there
+	are only small blocks on the free list this guarantees that the
+	memory for the pool is at the end of the memory which means that
+	we can give it back to the system when the pool is freed.
 */
 
-#if defined(HAVE_SYS_MMAN_H) && defined(HAVE_MMAP) && defined(HAVE_MUNMAP)
-
-#include <sys/mman.h>
-
-#if defined(MAP_ANONYMOUS) && defined(MAP_PRIVATE)
-
-#define USE_MMAP 1
-#define MMAP_FLAGS (MAP_ANONYMOUS | MAP_PRIVATE)
-
-#endif
-#endif
-
 #ifdef ZSH_MEM_WARNING
 # ifndef DEBUG
 #  define DEBUG 1
@@ -95,174 +84,122 @@
 
 #if defined(ZSH_MEM) && defined(ZSH_MEM_DEBUG)
 
-static int h_m[1025], h_push, h_pop, h_free;
+int h_m[1025], h_push, h_pop, h_free;
 
 #endif
 
-/* Make sure we align to the longest fundamental type. */
-union mem_align {
-    zlong l;
-    double d;
-};
-
-#define H_ISIZE  sizeof(union mem_align)
-#define HEAPSIZE (16384 - H_ISIZE)
-/* Memory available for user data in default arena size */
+#define H_ISIZE  sizeof(zlong)
+#define HEAPSIZE (8192 - H_ISIZE)
 #define HEAP_ARENA_SIZE (HEAPSIZE - sizeof(struct heap))
 #define HEAPFREE (16384 - H_ISIZE)
 
-/* Memory available for user data in heap h */
-#define ARENA_SIZEOF(h) ((h)->size - sizeof(struct heap))
-
-/* list of zsh heaps */
-
-static Heap heaps;
-
-/* a heap with free space, not always correct (it will be the last heap
- * if that was newly allocated but it may also be another one) */
-
-static Heap fheap;
-
-/* Use new heaps from now on. This returns the old heap-list. */
+/* set default allocation to heap stack */
 
 /**/
-mod_export Heap
-new_heaps(void)
+int
+global_heapalloc(void)
 {
-    Heap h;
+    int luh = useheap;
 
-    queue_signals();
-    h = heaps;
-
-    fheap = heaps = NULL;
-    unqueue_signals();
-
-    return h;
+    alloc = hcalloc;
+    ncalloc = halloc;
+    useheap = 1;
+    return luh;
 }
 
-/* Re-install the old heaps again, freeing the new ones. */
+/* set default allocation to malloc() */
 
 /**/
-mod_export void
-old_heaps(Heap old)
+int
+global_permalloc(void)
 {
-    Heap h, n;
+    int luh = useheap;
 
-    queue_signals();
-    for (h = heaps; h; h = n) {
-	n = h->next;
-	DPUTS(h->sp, "BUG: old_heaps() with pushed heaps");
-#ifdef USE_MMAP
-	munmap((void *) h, h->size);
-#else
-	zfree(h, HEAPSIZE);
+    alloc = zcalloc;
+    ncalloc = zalloc;
+    useheap = 0;
+    return luh;
+}
+
+/* heappush saves the current heap state using this structure */
+
+struct heapstack {
+    struct heapstack *next;	/* next one in list for this heap */
+    size_t used;
+};
+
+/* A zsh heap. */
+
+struct heap {
+    struct heap *next;		/* next one                                  */
+    size_t used;		/* bytes used from the heap                  */
+    struct heapstack *sp;	/* used by pushheap() to save the value used */
+#ifdef ZSH_64_BIT_TYPE
+    size_t dummy;		/* Make sure sizeof(heap) is a multiple of 8 */
 #endif
-    }
-    heaps = old;
-    fheap = NULL;
-    unqueue_signals();
-}
+#define arena(X)	((char *) (X) + sizeof(struct heap))
+};
 
-/* Temporarily switch to other heaps (or back again). */
+/* list of zsh heap */
 
-/**/
-mod_export Heap
-switch_heaps(Heap new)
-{
-    Heap h;
-
-    queue_signals();
-    h = heaps;
-
-    heaps = new;
-    fheap = NULL;
-    unqueue_signals();
-
-    return h;
-}
+Heap heaps;
 
 /* save states of zsh heaps */
 
 /**/
-mod_export void
+void
 pushheap(void)
 {
     Heap h;
     Heapstack hs;
 
-    queue_signals();
-
 #if defined(ZSH_MEM) && defined(ZSH_MEM_DEBUG)
     h_push++;
 #endif
 
     for (h = heaps; h; h = h->next) {
-	DPUTS(!h->used, "BUG: empty heap");
 	hs = (Heapstack) zalloc(sizeof(*hs));
 	hs->next = h->sp;
 	h->sp = hs;
 	hs->used = h->used;
     }
-    unqueue_signals();
 }
 
 /* reset heaps to previous state */
 
 /**/
-mod_export void
+void
 freeheap(void)
 {
-    Heap h, hn, hl = NULL;
-
-    queue_signals();
+    Heap h;
 
 #if defined(ZSH_MEM) && defined(ZSH_MEM_DEBUG)
     h_free++;
 #endif
-
-    fheap = NULL;
-    for (h = heaps; h; h = hn) {
-	hn = h->next;
-	if (h->sp) {
+    for (h = heaps; h; h = h->next) {
 #ifdef ZSH_MEM_DEBUG
+	if (h->sp)
 	    memset(arena(h) + h->sp->used, 0xff, h->used - h->sp->used);
+	else
+	    memset(arena(h), 0xff, h->used);
 #endif
-	    h->used = h->sp->used;
-	    if (!fheap && h->used < ARENA_SIZEOF(h))
-		fheap = h;
-	    hl = h;
-	} else {
-#ifdef USE_MMAP
-	    munmap((void *) h, h->size);
-#else
-	    zfree(h, HEAPSIZE);
-#endif
-	}
+	h->used = h->sp ? h->sp->used : 0;
     }
-    if (hl)
-	hl->next = NULL;
-    else
-	heaps = NULL;
-
-    unqueue_signals();
 }
 
 /* reset heap to previous state and destroy state information */
 
 /**/
-mod_export void
+void
 popheap(void)
 {
     Heap h, hn, hl = NULL;
     Heapstack hs;
 
-    queue_signals();
-
 #if defined(ZSH_MEM) && defined(ZSH_MEM_DEBUG)
     h_pop++;
 #endif
 
-    fheap = NULL;
     for (h = heaps; h; h = hn) {
 	hn = h->next;
 	if ((hs = h->sp)) {
@@ -271,136 +208,63 @@ popheap(void)
 	    memset(arena(h) + hs->used, 0xff, h->used - hs->used);
 #endif
 	    h->used = hs->used;
-	    if (!fheap && h->used < ARENA_SIZEOF(h))
-		fheap = h;
 	    zfree(hs, sizeof(*hs));
 
 	    hl = h;
-	} else {
-#ifdef USE_MMAP
-	    munmap((void *) h, h->size);
-#else
+	} else
 	    zfree(h, HEAPSIZE);
-#endif
-	}
     }
     if (hl)
 	hl->next = NULL;
     else
 	heaps = NULL;
-
-    unqueue_signals();
-}
-
-#ifdef USE_MMAP
-/*
- * Utility function to allocate a heap area of at least *n bytes.
- * *n will be rounded up to the next page boundary.
- */
-static Heap
-mmap_heap_alloc(size_t *n)
-{
-    Heap h;
-    static size_t pgsz = 0;
-
-    if (!pgsz) {
-
-#ifdef _SC_PAGESIZE
-	pgsz = sysconf(_SC_PAGESIZE);     /* SVR4 */
-#else
-# ifdef _SC_PAGE_SIZE
-	pgsz = sysconf(_SC_PAGE_SIZE);    /* HPUX */
-# else
-	pgsz = getpagesize();
-# endif
-#endif
-
-	pgsz--;
-    }
-    *n = (*n + pgsz) & ~pgsz;
-    h = (Heap) mmap(NULL, *n, PROT_READ | PROT_WRITE,
-		    MMAP_FLAGS, -1, 0);
-    if (h == ((Heap) -1)) {
-	zerr("fatal error: out of heap memory");
-	exit(1);
-    }
-
-    return h;
-}
-#endif
-
-/* check whether a pointer is within a memory pool */
-
-/**/
-mod_export void *
-zheapptr(void *p)
-{
-    Heap h;
-    queue_signals();
-    for (h = heaps; h; h = h->next)
-	if ((char *)p >= arena(h) &&
-	    (char *)p + H_ISIZE < arena(h) + ARENA_SIZEOF(h))
-	    break;
-    unqueue_signals();
-    return (h ? p : 0);
 }
 
 /* allocate memory from the current memory pool */
 
 /**/
-mod_export void *
-zhalloc(size_t size)
+void *
+halloc(size_t size)
 {
     Heap h;
     size_t n;
 
     size = (size + H_ISIZE - 1) & ~(H_ISIZE - 1);
 
-    queue_signals();
-
 #if defined(ZSH_MEM) && defined(ZSH_MEM_DEBUG)
-    h_m[size < (1024 * H_ISIZE) ? (size / H_ISIZE) : 1024]++;
+    h_m[size < 1024 ? (size / H_ISIZE) : 1024]++;
 #endif
 
     /* find a heap with enough free space */
 
-    for (h = ((fheap && ARENA_SIZEOF(fheap) >= (size + fheap->used))
-	      ? fheap : heaps);
-	 h; h = h->next) {
-	if (ARENA_SIZEOF(h) >= (n = size + h->used)) {
-	    void *ret;
-
+    for (h = heaps; h; h = h->next) {
+	if (HEAP_ARENA_SIZE >= (n = size + h->used)) {
 	    h->used = n;
-	    ret = arena(h) + n - size;
-	    unqueue_signals();
-	    return ret;
+	    return arena(h) + n - size;
 	}
     }
+
     {
 	Heap hp;
         /* not found, allocate new heap */
-#if defined(ZSH_MEM) && !defined(USE_MMAP)
+#ifdef ZSH_MEM
 	static int called = 0;
 	void *foo = called ? (void *)malloc(HEAPFREE) : NULL;
             /* tricky, see above */
 #endif
 
-	n = HEAP_ARENA_SIZE > size ? HEAPSIZE : size + sizeof(*h);
+	queue_signals();
+	n = HEAP_ARENA_SIZE > size ? HEAP_ARENA_SIZE : size;
 	for (hp = NULL, h = heaps; h; hp = h, h = h->next);
 
-#ifdef USE_MMAP
-	h = mmap_heap_alloc(&n);
-#else
-	h = (Heap) zalloc(n);
-#endif
+	h = (Heap) zalloc(n + sizeof(*h));
 
-#if defined(ZSH_MEM) && !defined(USE_MMAP)
+#ifdef ZSH_MEM
 	if (called)
 	    zfree(foo, HEAPFREE);
 	called = 1;
 #endif
 
-	h->size = n;
 	h->used = size;
 	h->next = NULL;
 	h->sp = NULL;
@@ -409,200 +273,66 @@ zhalloc(size_t size)
 	    hp->next = h;
 	else
 	    heaps = h;
-	fheap = h;
 
 	unqueue_signals();
 	return arena(h);
     }
 }
 
-/**/
-mod_export void *
-hrealloc(char *p, size_t old, size_t new)
-{
-    Heap h, ph;
-
-    old = (old + H_ISIZE - 1) & ~(H_ISIZE - 1);
-    new = (new + H_ISIZE - 1) & ~(H_ISIZE - 1);
-
-    if (old == new)
-	return p;
-    if (!old && !p)
-	return zhalloc(new);
-
-    /* find the heap with p */
-
-    queue_signals();
-    for (h = heaps, ph = NULL; h; ph = h, h = h->next)
-	if (p >= arena(h) && p < arena(h) + ARENA_SIZEOF(h))
-	    break;
-
-    DPUTS(!h, "BUG: hrealloc() called for non-heap memory.");
-    DPUTS(h->sp && arena(h) + h->sp->used > p,
-	  "BUG: hrealloc() wants to realloc pushed memory");
-
-    /*
-     * If the end of the old chunk is before the used pointer,
-     * more memory has been zhalloc'ed afterwards.
-     * We can't tell if that's still in use, obviously, since
-     * that's the whole point of heap memory.
-     * We have no choice other than to grab some more memory
-     * somewhere else and copy in the old stuff.
-     */
-    if (p + old < arena(h) + h->used) {
-	if (new > old) {
-	    char *ptr = (char *) zhalloc(new);
-	    memcpy(ptr, p, old);
-#ifdef ZSH_MEM_DEBUG
-	    memset(p, 0xff, old);
-#endif
-	    unqueue_signals();
-	    return ptr;
-	} else {
-	    unqueue_signals();
-	    return new ? p : NULL;
-	}
-    }
-
-    DPUTS(p + old != arena(h) + h->used, "BUG: hrealloc more than allocated");
-
-    /*
-     * We now know there's nothing afterwards in the heap, now see if
-     * there's nothing before.  Then we can reallocate the whole thing.
-     * Otherwise, we need to keep the stuff at the start of the heap,
-     * then allocate a new one too; this is handled below.  (This will
-     * guarantee we occupy a full heap next time round, provided we
-     * don't use the heap for anything else.)
-     */
-    if (p == arena(h)) {
-	/*
-	 * Zero new seems to be a special case saying we've finished
-	 * with the specially reallocated memory, see scanner() in glob.c.
-	 */
-	if (!new) {
-	    if (ph)
-		ph->next = h->next;
-	    else
-		heaps = h->next;
-	    fheap = NULL;
-#ifdef USE_MMAP
-	    munmap((void *) h, h->size);
-#else
-	    zfree(h, HEAPSIZE);
-#endif
-	    unqueue_signals();
-	    return NULL;
-	}
-	if (new > ARENA_SIZEOF(h)) {
-	    /*
-	     * Not enough memory in this heap.  Allocate a new
-	     * one of sufficient size.
-	     *
-	     * To avoid this happening too often, allocate
-	     * chunks in multiples of HEAPSIZE.
-	     * (Historical note:  there didn't used to be any
-	     * point in this since we didn't consistently record
-	     * the allocated size of the heap, but now we do.)
-	     */
-	    size_t n = (new + sizeof(*h) + HEAPSIZE);
-	    n -= n % HEAPSIZE;
-	    fheap = NULL;
-
-#ifdef USE_MMAP
-	    {
-		/*
-		 * I don't know any easy portable way of requesting
-		 * a mmap'd segment be extended, so simply allocate
-		 * a new one and copy.
-		 */
-		Heap hnew;
-
-		hnew = mmap_heap_alloc(&n);
-		/* Copy the entire heap, header (with next pointer) included */
-		memcpy(hnew, h, h->size);
-		munmap((void *)h, h->size);
-		h = hnew;
-	    }
-#else
-	    h = (Heap) realloc(h, n);
-#endif
-
-	    h->size = n;
-	    if (ph)
-		ph->next = h;
-	    else
-		heaps = h;
-	}
-	h->used = new;
-	unqueue_signals();
-	return arena(h);
-    }
-#ifndef USE_MMAP
-    DPUTS(h->used > ARENA_SIZEOF(h), "BUG: hrealloc at invalid address");
-#endif
-    if (h->used + (new - old) <= ARENA_SIZEOF(h)) {
-	h->used += new - old;
-	unqueue_signals();
-	return p;
-    } else {
-	char *t = zhalloc(new);
-	memcpy(t, p, old > new ? new : old);
-	h->used -= old;
-#ifdef ZSH_MEM_DEBUG
-	memset(p, 0xff, old);
-#endif
-	unqueue_signals();
-	return t;
-    }
-}
-
 /* allocate memory from the current memory pool and clear it */
 
 /**/
-mod_export void *
+void *
 hcalloc(size_t size)
 {
     void *ptr;
 
-    ptr = zhalloc(size);
+    ptr = halloc(size);
     memset(ptr, 0, size);
     return ptr;
 }
 
+/**/
+void *
+hrealloc(char *p, int old, int new)
+{
+    char *ptr;
+
+    ptr = (char *)halloc(new);
+    memcpy(ptr, p, old);
+    return (void *) ptr;
+}
+
 /* allocate permanent memory */
 
 /**/
-mod_export void *
+void *
 zalloc(size_t size)
 {
     void *ptr;
 
     if (!size)
 	size = 1;
-    queue_signals();
     if (!(ptr = (void *) malloc(size))) {
-	zerr("fatal error: out of memory");
+	zerr("fatal error: out of memory", NULL, 0);
 	exit(1);
     }
-    unqueue_signals();
 
     return ptr;
 }
 
 /**/
-mod_export void *
-zshcalloc(size_t size)
+void *
+zcalloc(size_t size)
 {
     void *ptr;
 
     if (!size)
 	size = 1;
-    queue_signals();
     if (!(ptr = (void *) malloc(size))) {
-	zerr("fatal error: out of memory");
+	zerr("fatal error: out of memory", NULL, 0);
 	exit(1);
     }
-    unqueue_signals();
     memset(ptr, 0, size);
 
     return ptr;
@@ -615,47 +345,68 @@ zshcalloc(size_t size)
  * POSIX compliant, but I'm not sure how to do that.                */
 
 /**/
-mod_export void *
+void *
 zrealloc(void *ptr, size_t size)
 {
-    queue_signals();
     if (ptr) {
 	if (size) {
 	    /* Do normal realloc */
 	    if (!(ptr = (void *) realloc(ptr, size))) {
-		zerr("fatal error: out of memory");
+		zerr("fatal error: out of memory", NULL, 0);
 		exit(1);
 	    }
-	    unqueue_signals();
 	    return ptr;
 	}
 	else
 	    /* If ptr is not NULL, but size is zero, *
 	     * then object pointed to is freed.      */
 	    free(ptr);
-
-	ptr = NULL;
     } else {
 	/* If ptr is NULL, then behave like malloc */
-	ptr = malloc(size);
+	return malloc(size);
     }
-    unqueue_signals();
 
-    return ptr;
+    return NULL;
 }
 
 /**/
+char *
+dupstring(const char *s)
+{
+    char *t;
+
+    if (!s)
+	return NULL;
+    t = (char *)ncalloc(strlen((char *)s) + 1);
+    strcpy(t, s);
+    return t;
+}
+
+/**/
+char *
+ztrdup(const char *s)
+{
+    char *t;
+
+    if (!s)
+	return NULL;
+    t = (char *)zalloc(strlen((char *)s) + 1);
+    strcpy(t, s);
+    return t;
+}
+
 #ifdef ZSH_MEM
 
 /*
    Below is a simple segment oriented memory allocator for systems on
    which it is better than the system's one. Memory is given in blocks
-   aligned to an integer multiple of sizeof(union mem_align), which will
-   probably be 64-bit as it is the longer of zlong or double. Each block is
-   preceded by a header which contains the length of the data part (in
-   bytes). In allocated blocks only this field of the structure m_hdr is
-   senseful. In free blocks the second field (next) is a pointer to the next
-   free segment on the free list.
+   aligned to an integer multiple of sizeof(zlong) (4 bytes on most machines,
+   but 8 bytes on e.g. a dec alpha, or if we are using 64-bit integer
+   arithmetic on a 32-bit machine). Each block is preceded by a header
+   which contains the length of the data part (in bytes). In allocated
+   blocks only this field of the structure m_hdr is senseful. In free
+   blocks the second field (next) is a pointer to the next free segment
+   on the free list.
 
    On top of this simple allocator there is a second allocator for small
    chunks of data. It should be both faster and less space-consuming than
@@ -711,7 +462,7 @@ zrealloc(void *ptr, size_t size)
 
 struct m_shdr {
     struct m_shdr *next;	/* next one on free list */
-#ifdef PAD_64_BIT
+#ifdef ZSH_64_BIT_TYPE
     /* dummy to make this 64-bit aligned */
     struct m_shdr *dummy;
 #endif
@@ -719,25 +470,18 @@ struct m_shdr {
 
 struct m_hdr {
     zlong len;			/* length of memory block */
-#if defined(PAD_64_BIT) && !defined(ZSH_64_BIT_TYPE)
-    /* either 1 or 2 zlong's, whichever makes up 64 bits. */
-    zlong dummy1;
-#endif
     struct m_hdr *next;		/* if free: next on free list
 				   if block of small blocks: next one with
 				                 small blocks of same size*/
     struct m_shdr *free;	/* if block of small blocks: free list */
     zlong used;			/* if block of small blocks: number of used
 				                                     blocks */
-#if defined(PAD_64_BIT) && !defined(ZSH_64_BIT_TYPE)
-    zlong dummy2;
-#endif
 };
 
 
 /* alignment for memory blocks */
 
-#define M_ALIGN (sizeof(union mem_align))
+#define M_ALIGN (sizeof(zlong))
 
 /* length of memory header, length of first field of memory header and
    minimal size of a block left free (if we allocate memory and take a
@@ -746,37 +490,23 @@ struct m_hdr {
    the free list) */
 
 #define M_HSIZE (sizeof(struct m_hdr))
-#if defined(PAD_64_BIT) && !defined(ZSH_64_BIT_TYPE)
-# define M_ISIZE (2*sizeof(zlong))
-#else
-# define M_ISIZE (sizeof(zlong))
-#endif
+#define M_ISIZE (sizeof(zlong))
 #define M_MIN   (2 * M_ISIZE)
 
-/* M_FREE  is the number of bytes that have to be free before memory is
- *         given back to the system
- * M_KEEP  is the number of bytes that will be kept when memory is given
- *         back; note that this has to be less than M_FREE
- * M_ALLOC is the number of extra bytes to request from the system */
-
-#define M_FREE  32768
-#define M_KEEP  16384
-#define M_ALLOC M_KEEP
-
 /* a pointer to the last free block, a pointer to the free list (the blocks
    on this list are kept in order - lowest address first) */
 
-static struct m_hdr *m_lfree, *m_free;
+struct m_hdr *m_lfree, *m_free;
 
 /* system's pagesize */
 
-static long m_pgsz = 0;
+long m_pgsz = 0;
 
 /* the highest and the lowest valid memory addresses, kept for fast validity
    checks in free() and to find out if and when we can give memory back to
    the system */
 
-static char *m_high, *m_low;
+char *m_high, *m_low;
 
 /* Management of blocks for small blocks:
    Such blocks are kept in lists (one list for each of the sizes that are
@@ -786,37 +516,29 @@ static char *m_high, *m_low;
    size of the small blocks held in a memory block, given a pointer to the
    header of it.  M_SBLEN() gives the size of a memory block that can hold
    an array of small blocks, given the size of these small blocks.  M_BSLEN()
-   calculates the size of the small blocks held in a memory block, given the
+   caculates the size of the small blocks held in a memory block, given the
    length of that block (including the header of the memory block.  M_NSMALL
    is the number of possible block sizes that small blocks should be used
    for. */
 
 
 #define M_SIDX(S)  ((S) / M_ISIZE)
-#define M_SNUM     128
+#define M_SNUM     50
 #define M_SLEN(M)  ((M)->len / M_SNUM)
-#if defined(PAD_64_BIT) && !defined(ZSH_64_BIT_TYPE)
-/* Include the dummy in the alignment */
-#define M_SBLEN(S) ((S) * M_SNUM + sizeof(struct m_shdr *) +  \
-		    2*sizeof(zlong) + sizeof(struct m_hdr *))
-#define M_BSLEN(S) (((S) - sizeof(struct m_shdr *) -  \
-		     2*sizeof(zlong) - sizeof(struct m_hdr *)) / M_SNUM)
-#else
 #define M_SBLEN(S) ((S) * M_SNUM + sizeof(struct m_shdr *) +  \
 		    sizeof(zlong) + sizeof(struct m_hdr *))
 #define M_BSLEN(S) (((S) - sizeof(struct m_shdr *) -  \
 		     sizeof(zlong) - sizeof(struct m_hdr *)) / M_SNUM)
-#endif
-#define M_NSMALL    8
+#define M_NSMALL 8
 
-static struct m_hdr *m_small[M_NSMALL];
+struct m_hdr *m_small[M_NSMALL];
 
 #ifdef ZSH_MEM_DEBUG
 
-static int m_s = 0, m_b = 0;
-static int m_m[1025], m_f[1025];
+int m_s = 0, m_b = 0;
+int m_m[1025], m_f[1025];
 
-static struct m_hdr *m_l;
+struct m_hdr *m_l;
 
 #endif /* ZSH_MEM_DEBUG */
 
@@ -825,31 +547,13 @@ malloc(MALLOC_ARG_T size)
 {
     struct m_hdr *m, *mp, *mt;
     long n, s, os = 0;
-#ifndef USE_MMAP
     struct heap *h, *hp, *hf = NULL, *hfp = NULL;
-#endif
 
     /* some systems want malloc to return the highest valid address plus one
-       if it is called with an argument of zero.
-    
-       TODO: really?  Suppose we allocate more memory, so
-       that this is now in bounds, then a more rational application
-       that thinks it can free() anything it malloc'ed, even
-       of zero length, calls free for it?  Aren't we in big
-       trouble?  Wouldn't it be safer just to allocate some
-       memory anyway?
-
-       If the above comment is really correct, then at least
-       we need to check in free() if we're freeing memory
-       at m_high.
-    */
+       if it is called with an argument of zero */
 
     if (!size)
-#if 1
-	size = 1;
-#else
 	return (MALLOC_RET_T) m_high;
-#endif
 
     queue_signals();  /* just queue signals rather than handling them */
 
@@ -916,14 +620,12 @@ malloc(MALLOC_ARG_T size)
     } else
 	s = 0;
 
-    /* search the free list for an block of at least the requested size */
+/* search the free list for an block of at least the requested size */
     for (mp = NULL, m = m_free; m && m->len < size; mp = m, m = m->next);
 
-#ifndef USE_MMAP
-
-    /* if there is an empty zsh heap at a lower address we steal it and take
-       the memory from it, putting the rest on the free list (remember
-       that the blocks on the free list are ordered) */
+ /* if there is an empty zsh heap at a lower address we steal it and take
+    the memory from it, putting the rest on the free list (remember
+    that the blocks on the free list are ordered) */
 
     for (hp = NULL, h = heaps; h; hp = h, h = h->next)
 	if (!h->used &&
@@ -950,15 +652,14 @@ malloc(MALLOC_ARG_T size)
 
 	for (mp = NULL, m = m_free; m && m->len < size; mp = m, m = m->next);
     }
-#endif
     if (!m) {
 	long nal;
 	/* no matching free block was found, we have to request new
 	   memory from the system */
-	n = (size + M_HSIZE + M_ALLOC + m_pgsz - 1) & ~(m_pgsz - 1);
+	n = (size + M_HSIZE + m_pgsz - 1) & ~(m_pgsz - 1);
 
 	if (((char *)(m = (struct m_hdr *)sbrk(n))) == ((char *)-1)) {
-	    DPUTS1(1, "MEM: allocation error at sbrk, size %L.", n);
+	    DPUTS(1, "allocation error at sbrk.");
 	    unqueue_signals();
 	    return NULL;
 	}
@@ -1064,7 +765,7 @@ malloc(MALLOC_ARG_T size)
    0 for this parameter means: `don't know' */
 
 /**/
-mod_export void
+void
 zfree(void *p, int sz)
 {
     struct m_hdr *m = (struct m_hdr *)(((char *)p) - M_ISIZE), *mp, *mt = NULL;
@@ -1085,7 +786,7 @@ zfree(void *p, int sz)
     /* first a simple check if the given address is valid */
     if (((char *)p) < m_low || ((char *)p) > m_high ||
 	((long)p) & (M_ALIGN - 1)) {
-	DPUTS(1, "BUG: attempt to free storage at invalid address");
+	DPUTS(1, "attempt to free storage at invalid address");
 	return;
     }
 
@@ -1117,7 +818,7 @@ zfree(void *p, int sz)
 		if ((((char *)p) - (((char *)mt) + sizeof(struct m_hdr))) %
 		    M_BSLEN(mt->len)) {
 
-		    DPUTS(1, "BUG: attempt to free storage at invalid address");
+		    DPUTS(1, "attempt to free storage at invalid address");
 		    unqueue_signals();
 		    return;
 		}
@@ -1125,13 +826,13 @@ zfree(void *p, int sz)
 		for (sh2 = mt->free; sh2; sh2 = sh2->next)
 		    if (((char *)p) == ((char *)sh2)) {
 
-			DPUTS(1, "BUG: attempt to free already free storage");
+			DPUTS(1, "attempt to free already free storage");
 			unqueue_signals();
 			return;
 		    }
 #endif
 		DPUTS(M_BSLEN(mt->len) < osz,
-		      "BUG: attempt to free more than allocated.");
+		      "attempt to free more than allocated.");
 
 #ifdef ZSH_MEM_DEBUG
 		m_f[M_BSLEN(mt->len) / M_ISIZE]++;
@@ -1191,7 +892,7 @@ zfree(void *p, int sz)
 
     /* no block was found at the given address */
     if (((char *)mt) >= m_high) {
-	DPUTS(1, "BUG: attempt to free storage at invalid address");
+	DPUTS(1, "attempt to free storage at invalid address");
 	unqueue_signals();
 	return;
     }
@@ -1202,11 +903,11 @@ zfree(void *p, int sz)
 
     if (m == mt) {
 	/* it is, ouch! */
-	DPUTS(1, "BUG: attempt to free already free storage");
+	DPUTS(1, "attempt to free already free storage");
 	unqueue_signals();
 	return;
     }
-    DPUTS(m->len < osz, "BUG: attempt to free more than allocated");
+    DPUTS(m->len < osz, "attempt to free more than allocated.");
 #ifdef ZSH_MEM_DEBUG
     memset(p, 0xff, m->len);
 #endif
@@ -1242,18 +943,12 @@ zfree(void *p, int sz)
        and now there is more than one page size of memory, we can give
        it back to the system (and we do it ;-) */
     if ((((char *)m_lfree) + M_ISIZE + m_lfree->len) == m_high &&
-	m_lfree->len >= m_pgsz + M_MIN + M_FREE) {
-	long n = (m_lfree->len - M_MIN - M_KEEP) & ~(m_pgsz - 1);
+	m_lfree->len >= m_pgsz + M_MIN) {
+	long n = (m_lfree->len - M_MIN) & ~(m_pgsz - 1);
 
 	m_lfree->len -= n;
-#ifdef HAVE_BRK
-	if (brk(m_high -= n) == -1) {
-#else
-	m_high -= n;
-	if (sbrk(-n) == (void *)-1) {
-#endif /* HAVE_BRK */
-	    DPUTS(1, "MEM: allocation error at brk.");
-	}
+	if (brk(m_high -= n) == -1)
+	    DPUTS(1, "allocation error at brk.");
 
 #ifdef ZSH_MEM_DEBUG
 	m_b += n;
@@ -1276,7 +971,7 @@ free(FREE_ARG_T p)
    those that have a zero byte at the end) */
 
 /**/
-mod_export void
+void
 zsfree(char *p)
 {
     if (p)
@@ -1347,55 +1042,50 @@ calloc(MALLOC_ARG_T n, MALLOC_ARG_T size)
 
 /**/
 int
-bin_mem(char *name, char **argv, Options ops, int func)
+bin_mem(char *name, char **argv, char *ops, int func)
 {
     int i, ii, fi, ui, j;
     struct m_hdr *m, *mf, *ms;
     char *b, *c, buf[40];
-    long u = 0, f = 0, to, cu;
+    long u = 0, f = 0;
 
-    queue_signals();
-    if (OPT_ISSET(ops,'v')) {
+    if (ops['v']) {
 	printf("The lower and the upper addresses of the heap. Diff gives\n");
 	printf("the difference between them, i.e. the size of the heap.\n\n");
     }
     printf("low mem %ld\t high mem %ld\t diff %ld\n",
 	   (long)m_l, (long)m_high, (long)(m_high - ((char *)m_l)));
 
-    if (OPT_ISSET(ops,'v')) {
+    if (ops['v']) {
 	printf("\nThe number of bytes that were allocated using sbrk() and\n");
 	printf("the number of bytes that were given back to the system\n");
 	printf("via brk().\n");
     }
     printf("\nsbrk %d\tbrk %d\n", m_s, m_b);
 
-    if (OPT_ISSET(ops,'v')) {
+    if (ops['v']) {
 	printf("\nInformation about the sizes that were allocated or freed.\n");
 	printf("For each size that were used the number of mallocs and\n");
 	printf("frees is shown. Diff gives the difference between these\n");
 	printf("values, i.e. the number of blocks of that size that is\n");
 	printf("currently allocated. Total is the product of size and diff,\n");
 	printf("i.e. the number of bytes that are allocated for blocks of\n");
-	printf("this size. The last field gives the accumulated number of\n");
-	printf("bytes for all sizes.\n");
+	printf("this size.\n");
     }
-    printf("\nsize\tmalloc\tfree\tdiff\ttotal\tcum\n");
-    for (i = 0, cu = 0; i < 1024; i++)
-	if (m_m[i] || m_f[i]) {
-	    to = (long) i * M_ISIZE * (m_m[i] - m_f[i]);
-	    printf("%ld\t%d\t%d\t%d\t%ld\t%ld\n",
-		   (long)i * M_ISIZE, m_m[i], m_f[i], m_m[i] - m_f[i],
-		   to, (cu += to));
-	}
+    printf("\nsize\tmalloc\tfree\tdiff\ttotal\n");
+    for (i = 0; i < 1024; i++)
+	if (m_m[i] || m_f[i])
+	    printf("%ld\t%d\t%d\t%d\t%ld\n", (long)i * M_ISIZE, m_m[i], m_f[i],
+		   m_m[i] - m_f[i], (long)i * M_ISIZE * (m_m[i] - m_f[i]));
 
     if (m_m[i] || m_f[i])
 	printf("big\t%d\t%d\t%d\n", m_m[i], m_f[i], m_m[i] - m_f[i]);
 
-    if (OPT_ISSET(ops,'v')) {
+    if (ops['v']) {
 	printf("\nThe list of memory blocks. For each block the following\n");
 	printf("information is shown:\n\n");
 	printf("num\tthe number of this block\n");
-	printf("tnum\tlike num but counted separately for used and free\n");
+	printf("tnum\tlike num but counted separatedly for used and free\n");
 	printf("\tblocks\n");
 	printf("addr\tthe address of this block\n");
 	printf("len\tthe length of the block\n");
@@ -1404,13 +1094,13 @@ bin_mem(char *name, char **argv, Options ops, int func)
 	printf("\t  free\tthis block is free\n");
 	printf("\t  small\tthis block is used for an array of small blocks\n");
 	printf("cum\tthe accumulated sizes of the blocks, counted\n");
-	printf("\tseparately for used and free blocks\n");
+	printf("\tseparatedly for used and free blocks\n");
 	printf("\nFor blocks holding small blocks the number of free\n");
 	printf("blocks, the number of used blocks and the size of the\n");
 	printf("blocks is shown. For otherwise used blocks the first few\n");
 	printf("bytes are shown as an ASCII dump.\n");
     }
-    printf("\nblock list:\nnum\ttnum\taddr\t\tlen\tstate\tcum\n");
+    printf("\nblock list:\nnum\ttnum\taddr\tlen\tstate\tcum\n");
     for (m = m_l, mf = m_free, ii = fi = ui = 1; ((char *)m) < m_high;
 	 m = (struct m_hdr *)(((char *)m) + M_ISIZE + m->len), ii++) {
 	for (j = 0, ms = NULL; j < M_NSMALL && !ms; j++)
@@ -1423,7 +1113,7 @@ bin_mem(char *name, char **argv, Options ops, int func)
 	else if (m == ms)
 	    sprintf(buf, "%ld %ld %ld", (long)(M_SNUM - ms->used),
 		    (long)ms->used,
-		    (long)(m->len - sizeof(struct m_hdr)) / M_SNUM + 1);
+		    (long)((m->len - sizeof(struct m_hdr)) / M_SNUM + 1));
 
 	else {
 	    for (i = 0, b = buf, c = (char *)&m->next; i < 20 && i < m->len;
@@ -1443,7 +1133,7 @@ bin_mem(char *name, char **argv, Options ops, int func)
 	    mf = mf->next;
     }
 
-    if (OPT_ISSET(ops,'v')) {
+    if (ops['v']) {
 	printf("\nHere is some information about the small blocks used.\n");
 	printf("For each size the arrays with the number of free and the\n");
 	printf("number of used blocks are shown.\n");
@@ -1455,14 +1145,13 @@ bin_mem(char *name, char **argv, Options ops, int func)
 	    printf("%ld\t", (long)i * M_ISIZE);
 
 	    for (ii = 0, m = m_small[i]; m; m = m->next) {
-		printf("(%ld/%ld) ", (long)(M_SNUM - m->used),
-		       (long)m->used);
+		printf("(%ld/%ld) ", (long)(M_SNUM - m->used), (long)m->used);
 		if (!((++ii) & 7))
 		    printf("\n\t");
 	    }
 	    putchar('\n');
 	}
-    if (OPT_ISSET(ops,'v')) {
+    if (ops['v']) {
 	printf("\n\nBelow is some information about the allocation\n");
 	printf("behaviour of the zsh heaps. First the number of times\n");
 	printf("pushheap(), popheap(), and freeheap() were called.\n");
@@ -1471,7 +1160,7 @@ bin_mem(char *name, char **argv, Options ops, int func)
 
     printf("push %d\tpop %d\tfree %d\n\n", h_push, h_pop, h_free);
 
-    if (OPT_ISSET(ops,'v')) {
+    if (ops['v']) {
 	printf("\nThe next list shows for several sizes the number of times\n");
 	printf("memory of this size were taken from heaps.\n\n");
     }
@@ -1483,30 +1172,27 @@ bin_mem(char *name, char **argv, Options ops, int func)
     if (h_m[1024])
 	printf("big\t%d\n", h_m[1024]);
 
-    unqueue_signals();
     return 0;
 }
 
 #endif
 
-/**/
 #else				/* not ZSH_MEM */
 
 /**/
-mod_export void
-zfree(void *p, UNUSED(int sz))
+void
+zfree(void *p, int sz)
 {
     if (p)
 	free(p);
 }
 
 /**/
-mod_export void
+void
 zsfree(char *p)
 {
     if (p)
 	free(p);
 }
 
-/**/
 #endif
